import os
from ..conf.conf import config
from .file import load_project_config

def ensure_component_exists(name) :
    config = load_project_config()
    components = [ component['name'] for component in config['components'] ]
    if name in components :
        return  True
    return False

def ensure_abrio_root() :
    path = os.getcwd()
    file = config['abrio_root_file']
    if os.path.exists(os.path.join(path, file)) :
        return True
    return False



import click

import component.component as Component
import project.project as Project




@click.group()
def cli():
    '''
    Abrio Core Command Line Interface

        .________        _______       ______        ________      ______
        /_______/\     /_______/\     /_____/\      /_______/\    /_____/\
        \::: _  \ \    \::: _  \ \    \:::_ \ \     \__.::._\/    \:::_ \ \
        .\::(_)  \ \    \::(_)  \/_    \:(_) ) )_      \::\ \      \:\ \ \ \
        ..\:: __  \ \    \::  _  \ \    \: __ `\ \     _\::\ \__    \:\ \ \ \
        ...\:.\ \  \ \    \::(_)  \ \    \ \ `\ \ \   /__\::\__/\    \:\_\ \ \
        ....\__\/\__\/     \_______\/     \_\/ \_\/   \________\/     \_____\/  '''
    pass

# Croup Command for Project
@cli.group()
def project() :
    '''
    Create and manage a project
    '''
    pass


# Group Command for component
@cli.group()
def component() :
    '''
    Create and manage a component
    '''
    pass

project.add_command(Project.init)
project.add_command(Project.deploy)
project.add_command(Project.stop)
project.add_command(Project.status)

component.add_command(Component.init)
component.add_command(Component.upload)
component.add_command(Component.rm)
component.add_command(Component.ls)



import click, requests, json
from terminaltables import AsciiTable
from ..conf.conf import errors
from ..util.file import *
from ..util.checker import *

@click.command()
@click.option('--pkey', prompt="Enter Project private key (create one in website if you don't have one)",)
@click.option('--email', prompt="Enter your Abrio Account Email")
@click.option('--password', prompt="Enter your Abrio Account Password" , hide_input=True)

def init(pkey, email, password) :
    '''
    Create new Abrio Project
    '''
    config = {
        'email' : email,
        'password' : password,
        'private_key' : pkey,
        'components' : [],
        'create_date' : ''
    }

    write_project_config(config)

    click.secho('\ndirectory marked as abrio project root. happy coding.\n',bold=True, fg='green')


@click.command()
def deploy() :
    '''
    Deploy Abrio project
    '''
    if not ensure_abrio_root():
        click.secho('\nAbrio Root Directory Not Detected.\n', fg="red", bold=True)
        return
    pkey = load_project_config()['private_key']
    response = requests.post(
        config['server']['host'] + 'project/start',
        json={'private_key' : pkey }
    )

    if response.status_code == 200:
        click.secho("\nProject Successfully Lunched.\n", bold=True, fg='green')

    elif response.status_code == 409 :
        click.secho("\nProject already lunched.\n" , bold=True, fg="yellow")

    else:
        click.secho(errors["UNKNOWN_NETWORK"], bold=True, fg="red")


@click.command()
def stop() :
    '''
    Stop Abrio Project
    '''
    if not ensure_abrio_root():
        click.secho('\nAbrio Root Directory Not Detected.\n', fg="red", bold=True)
        return

    pkey = load_project_config()['private_key']
    response = requests.post(
        config['server']['host'] + 'project/stop',
        json={'private_key': pkey}
    )

    if response.status_code == 200:
        click.secho("\nProject Successfully Stopped.\n", bold=True, fg='green')

    else:
        click.secho(errors["UNKNOWN_NETWORK"], bold=True, fg="red")


@click.command()
def status() :
    '''
    View Abrio project status
    '''
    if not ensure_abrio_root():
        click.secho('\nAbrio Root Directory Not Detected.\n', fg="red", bold=True)
        return

    click.secho("\nConnecting to server..\n", fg='yellow', bold=True)
    project_config = load_project_config()
    pkey = project_config['private_key']
    response = requests.get(
        config['server']['host'] + 'project/status',
        json={'private_key': pkey}
    )
    if response.status_code == 200 :
        content = json.loads(response.content)
        project_config['create_date'] = content['create_date']
        project_config['name'] = content['name']
        write_project_config(project_config)

        project_table = [
            ['Name', "Is Running", "Created At", "Owner"],
            [
                project_config['name'],
                str(content['is_running']),
                project_config['create_date'],
                project_config['email']
            ]
        ]
        click.echo(AsciiTable(project_table).table)

    else :
        click.secho(errors["UNKNOWN_NETWORK"], bold=True, fg="red")




import json

def load_project_config() :
    '''
    read config dict from json file
    '''
    with open('./abrio.json', 'r') as config_file:
        return json.load(config_file)

def load_component_config(name) :
    '''
    shorthand for opening config file for one component only
    '''
    with open('./abrio.json', 'r') as config_file:
        project_config = json.load(config_file)
        for comp in project_config['components'] :
            if comp['name'] == name :
                return comp
    return False

def write_project_config(config) :
    '''
    write config dict to json file
    '''
    with open('./abrio.json', 'w' ) as config_file:
        config_file.write(json.dumps(config, indent=4, separators=(',', ': ')))

def write_component_config(name, component_config) :
    project_config = load_project_config()
    components = project_config['components']
    for idx,comp in enumerate(components) :
        if comp['name'] == name :
            components[idx] = component_config
            write_project_config(project_config)
            return

def add_component_project(component_config) :
    '''
    add a component to project components
    '''
    config = load_project_config()
    components = config['components']
    components.append(component_config)
    write_project_config(config)

def remove_component_project(name) :
    '''
    remove a component form project
    '''
    config = load_project_config()
    components = config['components']
    for comp in components :
        if comp['name'] == name :
            components.remove(comp)
            write_project_config(config)
            return


from setuptools import setup

setup(
    # Main Descriptions
    name="AbrIO",
    version="0.0.5",
    description = 'Abrio Engine cli for developers',
    author = 'AbrIO',
    author_email = 'abrioengine@gmail.com',
    url = 'https://github.com/kianenigma/abrio-cli',
    download_url = 'https://github.com/kianenigma/abrio-cli/archive/0.0.43.tar.gz',

    # Included packages
    packages=['abriocli', 'abriocli.component', 'abriocli.project', 'abriocli.util', 'abriocli.conf'],

    # Included Dependencies
    install_requires=[
        'Click',
        'clint',
        'requests_toolbelt',
        'terminaltables',
        'colorclass'
    ],

    # Application Entry Point . Probably Just this one
    entry_points={
        'console_scripts' : ['abrio=abriocli.abrio:cli']
    } ,

    # Metadata Configurations 
    include_package_data=True,
      zip_safe=False
)



config = {
    "server" : {
        # "host" : "http://127.0.0.1:5000/api/v1/",
        "host" : "http://abrio.ir/api/v1/",
    } ,
    "abrio_root_file" : "abrio.json",
    "sdk_file" : "AbrioSDK-V0.0.5.zip"
}

errors = {
    "UNKNOWN_NETWORK" : "Unknown Error occurred while connection to server. try again"
}

from os.path import *
MAIN_DIRECTORY = dirname(dirname(__file__))
def get_full_path(*path):
    return join(MAIN_DIRECTORY, *path)



import click, zipfile, json
import requests
import datetime
from colorclass import Color
from terminaltables import AsciiTable

from requests.auth import HTTPBasicAuth
from requests_toolbelt import MultipartEncoder, MultipartEncoderMonitor
from clint.textui.progress import Bar as ProgressBar

from ..util.file import *
from ..util.checker import *
from ..conf.conf import config, get_full_path
from ..conf.conf import config,errors


@click.option('--version', prompt="Enter component version", default="0.0.1" )
@click.option('--public', is_flag=True, prompt='Do you want to mark this component as public?' )
@click.option('--name', prompt="Enter component name", default="Test")
@click.command()

def init(name, public, version):
    '''
    Create new abrio component.
    '''
    if not ensure_abrio_root() :
        click.secho('\nAbrio Root Directory Not Detected.\n' , fg="red", bold=True)
        return

    if os.path.exists(name) :
        click.secho("\nDirecotry with name <{0}> already exists.\n".format(name), fg="red", bold=True)
        return

    click.secho("\nConnection to sever..." , bold=True, fg="yellow")
    project_config = load_project_config()
    email = project_config['email']
    pwd = project_config['password']
    name = name
    is_private = public
    response = requests.post(
        config['server']['host']+'component/create',
        auth=HTTPBasicAuth(email, pwd),
        json={'isPrivate': is_private, 'name': name}
    )

    if response.status_code == 201 :
        pkey = response.json()['token']
        os.mkdir(name)
        zip_file = zipfile.ZipFile(os.path.join(get_full_path('data', config['sdk_file'])))
        zip_file.extractall(name)

        component_config = {
            'pkey': pkey,
            'public': public,
            'version': version,
            'name': name,
            'last_uploaded': ''
        }

        # with open(os.path.join(name, (name+'.json')), 'w') as config_file :
        #     config_file.write(json.dumps(component_config, indent=4, separators=(',', ': ')))

        add_component_project(component_config)

        click.secho("\nComponent <{0}> created.\n".format(name), bold=True, fg='green')

    else :
        click.secho(errors["UNKNOWN_NETWORK"],bold=True, fg="red")


@click.command()
@click.argument('name')
def upload(name) :
    '''
    Upload Abrio component to server.
    '''

    if not ensure_abrio_root():
        click.secho('\nAbrio Root Directory Not Detected.\n', fg="red", bold=True)
        return

    if not ensure_component_exists(name):
        click.secho("\nComponent <{0}> does not exist.\n".format(name), bold=True, fg="red")

    build_dir = '/sample/build/libs/'

    os.system('cd {0} && gradle jar && cd ..'.format(name))
    jar_dir = name + build_dir + name + '.jar'
    os.rename(name + build_dir + 'sample.jar',jar_dir)

    encoder = create_upload(jar_dir)
    callback = create_callback(encoder)
    monitor = MultipartEncoderMonitor(encoder, callback)

    component_config = load_component_config(name)
    component_config['last_uploaded'] = str(datetime.datetime.now())
    write_component_config(name, component_config)

    headers = {
        'Content-Type': monitor.content_type,
        'private key': component_config['pkey'],
        'version' : component_config['version']
    }

    upload_response = requests.post(
        config['server']['host'] + "component/upload",
        data=monitor,
        # auth=HTTPBasicAuth(email, pwd),
        headers=headers)

    if upload_response.status_code == 200 :
        click.secho('\n\n\nComponent uploaded\n', bold=True, fg="green")

    else :
        click.secho(errors["UNKNOWN_NETWORK"], bold=True, fg="red")





@click.option('--sure', prompt="Are you sure you want to delete this component", is_flag=True)
@click.argument('name')
@click.command()
def rm(name, sure) :
    '''
    Delete Abrio Component.
    '''

    if not ensure_abrio_root():
        click.secho('\nAbrio Root Directory Not Detected.\n', fg="red", bold=True)
        return

    if sure :
        if ensure_component_exists(name) :
            os.system('rm -Rf {0}'.format(name))
            remove_component_project(name)
            # todo delete from server
            click.secho("\nComponent <{0}> deleted.\n".format(name), bold=True, fg="yellow")

        else :
            click.secho("\nComponent <{0}> does not exist.\n".format(name), bold=True, fg="red")

@click.command()
def ls() :
    '''
    List Available Abrio components
    '''

    if not ensure_abrio_root():
        click.secho('\nAbrio Root Directory Not Detected.\n', fg="red", bold=True)
        return



    project_config = load_project_config()

    response = requests.get(
        config['server']['host'] + "project/list_components",
        json={'private_key': project_config['private_key']}
    )

    if response.status_code == 200 :
        component_table = [
                        ['Component Name', 'Version', 'Public', "Last Upload" , "Type"]] + \
                        [
                            [
                                component['name'],
                                component['version'],
                                str(component['public']),
                                component['last_uploaded'],
                                Color('{autoyellow}Local{/autoyellow}')
                            ] for component in project_config['components']
                        ]

        component_table += [
            [
                comp['name'],
                comp['deploy_version'],
                 str(not comp['private']),
                 "---",
                 Color('{autocyan}Online{/autocyan}')
             ] for comp in json.loads(response.content)['result']]


        table = AsciiTable(component_table)
        click.echo(table.table)

    else :
        click.secho(errors["UNKNOWN_NETWORK"], bold=True, fg="red")




def create_callback(encoder):
    encoder_len = encoder.len

    bar = ProgressBar(expected_size=encoder_len, filled_char='=')

    def callback(monitor):
        bar.show(monitor.bytes_read)
    return callback

def create_upload(file_path):
    file_name = file_path.split("/")[-1]
    return MultipartEncoder({'files':(file_name,open(file_path, 'rb'))})